!	----------------------------------------------------------------------
!	MeasureModule.f90
!	----------------------------------------------------------------------


module MeasureModule

	use Globals
	use LinInterpModule
	

	implicit none


	real(8):: invAE(dna,nx), invAU(dna)


contains


subroutine InitializeDAgrid()


	!	construct equally spaced asset grids that is used to find invaraint measures

	step_a = (amax-amin)/(dna-1)

	dagrid(1) = amin
	do ia = 2, dna
		dagrid(ia) = dagrid(ia-1) + step_a
	end do


	! save dagrid

	open(1, file='Output\dagrid.txt', status='unknown')
	write(1, '(f12.6)') dagrid
	close(1)
	
	
end subroutine



subroutine InitializeMeasures()

	implicit none

	integer i, j

 
	!	initialize the cumulative measure of workers over assets

	do ia = 1, dna
		do ix = 1, nx
			mu(ia,ix) = prX(ix)*(dagrid(ia)-dagrid(1))/(dagrid(dna)-dagrid(1))
		end do
		phi(ia) = (dagrid(ia)-dagrid(1))/(dagrid(dna)-dagrid(1))
	end do

	mu  = 0.94D0*mu
	phi = 0.06D0*phi


end subroutine



subroutine InverseAsseetDecisions()


	!	compute inverse asset decision for employed workers

	do ix = 1, nx
	do ia = 1, dna

		if (dagrid(ia) <= AE(na,ix) .and. dagrid(ia) >= AE(1,ix)) then
			invAE(ia,ix) = lininterp1(dagrid(ia), AE(:,ix), agrid)
		else if (dagrid(ia) > AE(na,ix)) then
			invAE(ia,ix) = dagrid(dna)
		else if (dagrid(ia) < AE(1,ix)) then
			invAE(ia,ix) = dagrid(1)
		end if

	end do
	end do


	!	compute inverse asset decision for unemployed workers

	do ia = 1, dna

		if (dagrid(ia) <= AU(na) .and. dagrid(ia) >= AU(1)) then
			invAU(ia) = lininterp1(dagrid(ia), AU, agrid)
		else if (dagrid(ia) > AU(na)) then
			invAU(ia) = dagrid(dna)
		else if (dagrid(ia) < AU(1)) then
			invAU(ia) = dagrid(1)
		end if

	end do


end subroutine



subroutine InvariantMeasures()


	!	preparation for computing invariant measures

	call InitializeMeasures()
	call InverseAsseetDecisions()


	!	start iteration

	err_measure = 1.0D0
	iter_measure = 0

	do while (err_measure > tol_measure)


		!	measures after asset decisions

		muhat = 0.0D0
		phihat = 0.0D0

		do ix = 1, nx
		do ia = 1, dna
			muhat(ia,ix) = lininterp1(invAE(ia,ix), dagrid, mu(:,ix))
		end do
		end do

		do ia = 1, dna
			phihat(ia) = lininterp1(invAU(ia), dagrid, phi)
		end do


		!	make sure the total measure doesn't change

		do ix = 1, nx
			if (muhat(dna,ix) .ne. 0.0D0) then
				muhat(:,ix) = muhat(:,ix)/muhat(dna,ix)*mu(dna,ix)
			end if
		end do
		phihat = phihat/phihat(dna)*phi(dna)

		
		!	measures after new matching and exogenous separations

		do ia = 1, dna

			do ix = 1, nx
				mutilde(ia,ix) = (1.0D0-lambda)*dot_product(muhat(ia,:), trX(:,ix)) + pth*phihat(ia)*prX(ix)
			end do
	
			phitilde(ia) = lambda*sum(muhat(ia,:)) + (1.0D0-pth)*phihat(ia)

		end do


		!	new measure after endogenous match separations

		newmu = mutilde
		newphi = phitilde

		do ix = 1, nx

			if (cutoff_a(ix) == 1) then
				newmu(:,ix) = 0.0D0
				newphi = newphi + mutilde(:,ix)
			else if (cutoff_a(ix) <= dna) then
				newmu(cutoff_a(ix):,ix) = newmu(cutoff_a(ix)-1,ix)
				newphi(cutoff_a(ix):) = newphi(cutoff_a(ix):)		&
					+ (mutilde(cutoff_a(ix):,ix) - newmu(cutoff_a(ix)-1,ix))
			end if

		end do


		!	calculate the differences between old and new measures

		err_measure = max(maxval(dabs(newmu-mu)), maxval(dabs(newphi-phi)))


		!	update measures 

		mu = newmu
		phi = newphi

		iter_measure = iter_measure + 1


		!	show progress

		if (mod(iter_measure, 2000) == 0) then
			print*, "InvariantMeasure iteration", iter_measure
			print*, sum(mu(dna,:)), phi(dna)
			print*, "err_measure = ", err_measure
			print*, ""
		end if

	end do


	!	print the final iteration number

	print*, "Measures have converged at iteration", iter_measure
	print*, ""


	!	compute measures as densities

	do ia = dna, 2, -1
		do ix = 1, nx
			mu(ia,ix) = mu(ia,ix) - mu(ia-1,ix)
			muhat(ia,ix) = muhat(ia,ix) - muhat(ia-1,ix)
			mutilde(ia,ix) = mutilde(ia,ix) - mutilde(ia-1,ix)
		end do
		phi(ia) = phi(ia) - phi(ia-1)
		phihat(ia) = phihat(ia) - phihat(ia-1)
		phitilde(ia) = phitilde(ia) - phitilde(ia-1)
	end do
		

end subroutine



subroutine AssetEarningsDistribution()


	real(8):: earnstep
	integer:: earnindex
	
	
	!	earnings grid

	earnings(1) = minval(dwage, mu > 0.0D0)
	earnings(dnx) = maxval(dwage, mu > 0.0D0)
	earnstep = (earnings(dnx)-earnings(1))/(dnx-1)

	do ix = 2, dnx
		earnings(ix) = earnings(ix-1) + earnstep
	end do
	

	!	joint density for asset and earnings 

	AsstEarnDensity = 0.0D0
	AsstEarnDensity(:,0) = phi

	do ia = 1, dna
	do ix = 1, nx
		
		if (mu(ia,ix) == 0.0D0) then
			AsstEarnDensity(ia,0) = AsstEarnDensity(ia,0) + mu(ia,ix)
		else
			earnindex = locate(dwage(ia,ix), earnings)
			AsstEarnDensity(ia,earnindex) = AsstEarnDensity(ia,earnindex) + mu(ia,ix)
		end if

	end do
	end do


	!	marginal density for asset

	AsstDensity = sum(AsstEarnDensity,dim=2)


	!	marginal density for earnings

	EarnDensity = sum(AsstEarnDensity,dim=1)


	!	joint distribution for asset and earnings

	AsstEarnDist = AsstEarnDensity

	do ix = 1, dnx
		AsstEarnDist(:,ix) = AsstEarnDist(:,ix-1) + AsstEarnDensity(:,ix)
	end do

	do ia = 2, dna
		AsstEarnDist(ia,:) = AsstEarnDist(ia-1,:) + AsstEarnDist(ia,:)
	end do


	!	marginal distribution for asset

	AsstDist = AsstDensity

	do ia = 2, dna
		AsstDist(ia) = AsstDist(ia-1) + AsstDensity(ia)
	end do


	!	marginal distribution for earnings

	EarnDist = EarnDensity

	do ix = 1, dnx
		EarnDist(ix) = EarnDist(ix-1) + EarnDensity(ix)
	end do


end subroutine




end module