!	----------------------------------------------------------------------
!	File name: AggregateVariables.f90
!
!	This subroutine computes aggregate labor market variables 
!	in the steady state.
!	----------------------------------------------------------------------

subroutine AggregateVariables()

	use Globals

    
	implicit none


	real(8):: empl, unempl, EtoU, UtoE, hEtoU, hUtoE, empl_dur, unempl_dur,	&
			  ave_asset_E, ave_asset_U, std_asset_E, std_asset_U, ave_wage, std_wage, ave_lwage, std_lwage
	real(8):: cscoef_wage(4,na), dwage(dna)
	real(8), external:: csval
	

	!	employment

	empl = sum(mu)
	unempl = sum(phi)


	!	matching probabilities and vacancies

	pth = match_scale*theta**match_pow
	qth = pth/theta
	vac = theta*unempl


	!	worker flows, average tenure, average wage

	EtoU = lambda*empl
	UtoE = pth*unempl

	hEtoU = EtoU/empl
	hUtoE = UtoE/unempl
	empl_dur = 1.0D0/hEtoU
	unempl_dur = 1.0D0/hUtoE

	call csint(na, agrid, Wage, cscoef_wage)
	do ia = 1, dna
		dwage(ia) = csval(na, dagrid(ia), agrid, cscoef_wage)
	end do
	ave_wage = dot_product(dwage, mu/empl)
	std_wage = dsqrt(dot_product(dwage**2.0D0, mu/empl) - ave_wage**2.0D0)
    
    ave_lwage = dot_product(dlog(dwage), mu)/Empl
	std_lwage = dsqrt(sum(dlog(dwage)**2.0D0*mu/empl) - ave_lwage**2.0D0)
	

	!	wealth distribution

	ave_asset = dot_product(dagrid, mu+phi)
	ave_asset_E = dot_product(dagrid, mu/empl)
	ave_asset_U = dot_product(dagrid, phi/unempl)

	std_asset = dsqrt(dot_product(dagrid**2.0D0, mu+phi) - ave_asset**2.0D0)
	std_asset_E = dsqrt(dot_product(dagrid**2.0D0, mu/empl) - ave_asset_E**2.0D0)
	std_asset_U = dsqrt(dot_product(dagrid**2.0D0, phi/unempl) - ave_asset_U**2.0D0)


	!	show parameters on screen

	write(*, '(//)') 
	write(*, '(A)') "Model Parameters:"
	write(*, '(A,F12.8)') "alpha       = ", alpha
	write(*, '(A,F12.8)') "bta         = ", bta
	write(*, '(A,F12.8)') "gama        = ", gama
	write(*, '(A,F12.8)') "lambda      = ", lambda
	write(*, '(A,F12.8)') "Prod        = ", Prod
	write(*, '(A,F12.8)') "UI          = ", UI
	write(*, '(A,F12.8)') "B_leisure   = ", B_leisure
	write(*, '(A,F12.8)') "kappa       = ", kappa
	write(*, '(A,F12.8)') "match_pow   = ", match_pow
	write(*, '(A,F12.8)') "match_scale = ", match_scale
	write(*, '(/)') 


	!	show aggregate variables on screen

	write(*, '(A/)') "Aggregate Variables:"
	write(*, '(A,F12.8)') "theta                   = ", theta
	write(*, '(A,F12.8)') "p(theta)                = ", pth
	write(*, '(A,F12.8)') "q(theta)                = ", qth
	write(*, '(A,F12.8)') "vac                     = ", vac
	write(*, '(A)') ""
	write(*, '(A,F12.8)') "Employment              = ", empl
	write(*, '(A,F12.8)') "Unemployment            = ", unempl
	write(*, '(A)') ""
	write(*, '(A,F12.8)') "Worker flow EtoU        = ", EtoU
	write(*, '(A,F12.8)') "            UtoE        = ", UtoE
	write(*, '(A)') ""
	write(*, '(A,F12.8)') "Hazard rate EtoU        = ", hEtoU
	write(*, '(A,F12.8)') "            UtoE        = ", hUtoE
	write(*, '(A)') ""
	write(*, '(A,F12.8)') "Employment duration     = ", empl_dur
	write(*, '(A,F12.8)') "Unemployment duration   = ", unempl_dur
	write(*, '(A)') ""
	write(*, '(A,F12.8)') "Ave. Wage               = ", ave_wage
	write(*, '(A,F12.8)') "Std. Wage               = ", std_wage
	write(*, '(A,F12.8)') "Ave. Log Wage           = ", ave_lwage
	write(*, '(A,F12.8)') "Std. Log wage           = ", std_lwage
	write(*, '(A)') ""
	write(*, '(A,F12.8)') "Average Wealth          = ", ave_asset
	write(*, '(A,F12.8)') "Ave. Wealth (Empl)      = ", ave_asset_E
	write(*, '(A,F12.8)') "Ave. Wealth (Unempl)    = ", ave_asset_U
	write(*, '(A,F12.8)') "Std. Wealth             = ", std_asset
	write(*, '(A,F12.8)') "Std. Wealth (Empl)      = ", std_asset_E
	write(*, '(A,F12.8)') "Std. Wealth (Unempl)    = ", std_asset_U
	write(*, '(/)') 


	!	open an output file

	open(1, file='Output\Aggregates.txt', status='unknown')


	!	write parameters on file

	write(1, '(A)') "Model Parameters:"
	write(1, '(A,F12.8)') "alpha       = ", alpha
	write(1, '(A,F12.8)') "bta         = ", bta
	write(1, '(A,F12.8)') "gama        = ", gama
	write(1, '(A,F12.8)') "lambda      = ", lambda
	write(1, '(A,F12.8)') "Prod        = ", Prod
	write(1, '(A,F12.8)') "UI          = ", UI
	write(1, '(A,F12.8)') "B_leisure   = ", B_leisure
	write(1, '(A,F12.8)') "kappa       = ", kappa
	write(1, '(A,F12.8)') "match_pow   = ", match_pow
	write(1, '(A,F12.8)') "match_scale = ", match_scale
	write(1, '(/)') 


	!	write aggregate variables on file

	write(1, '(A/)') "Aggregate Variables:"
	write(1, '(A,F12.8)') "theta                   = ", theta
	write(1, '(A,F12.8)') "p(theta)                = ", pth
	write(1, '(A,F12.8)') "q(theta)                = ", qth
	write(1, '(A,F12.8)') "vac                     = ", vac
	write(1, '(A)') ""
	write(1, '(A,F12.8)') "Employment              = ", empl
	write(1, '(A,F12.8)') "Unemployment            = ", unempl
	write(1, '(A)') ""
	write(1, '(A,F12.8)') "Worker flow EtoU        = ", EtoU
	write(1, '(A,F12.8)') "            UtoE        = ", UtoE
	write(1, '(A)') ""
	write(1, '(A,F12.8)') "Hazard rate EtoU        = ", hEtoU
	write(1, '(A,F12.8)') "            UtoE        = ", hUtoE
	write(1, '(A)') ""
	write(1, '(A,F12.8)') "Employment duration     = ", empl_dur
	write(1, '(A,F12.8)') "Unemployment duration   = ", unempl_dur
	write(1, '(A)') ""
	write(1, '(A,F12.8)') "Ave. Wage              = ", ave_wage
	write(1, '(A,F12.8)') "Std. Wage              = ", std_wage
	write(1, '(A,F12.8)') "Ave. Log Wage           = ", ave_lwage
	write(1, '(A,F12.8)') "Std. Log wage           = ", std_lwage

	write(1, '(A)') ""
	write(1, '(A,F12.8)') "Average Wealth          = ", ave_asset
	write(1, '(A,F12.8)') "Ave. Wealth (Empl)      = ", ave_asset_E
	write(1, '(A,F12.8)') "Ave. Wealth (Unempl)    = ", ave_asset_U
	write(1, '(A,F12.8)') "Std. Wealth             = ", std_asset
	write(1, '(A,F12.8)') "Std. Wealth (Empl)      = ", std_asset_E
	write(1, '(A,F12.8)') "Std. Wealth (Unempl)    = ", std_asset_U
	write(1, '(/)') 


	!	set the finishing time

	call date_and_time(finishdate,finishtime)


	!	print the starting and finishing times

	write(1, '(/)') 
	write(1, '(12A)') "The program started at  ", startdate(1:4),"-",startdate(5:6),"-",startdate(7:8)," ", starttime(1:2),":",starttime(3:4),":",starttime(5:6)
	write(1, '(12A)') "The program finished at ", finishdate(1:4),"-",finishdate(5:6),"-",finishdate(7:8)," ", finishtime(1:2),":",finishtime(3:4),":",finishtime(5:6)
	close(1)


end subroutine
